@@ -3,6 +3,8 @@
 
 #define DEBUG_BIOSTRINGS 1
 
+#define INTERNAL_ERR_IN "Biostrings internal error in "
+
 #define INIT_STATIC_SYMBOL(NAME) \
 { \
 	if (NAME ## _symbol == NULL) \
@@ -27,9 +29,9 @@ void _init_byte2offset_with_INTEGER(
 	int error_on_dup
 );
 
-void _init_byte2offset_with_cachedCharSeq(
+void _init_byte2offset_with_Chars_holder(
 	ByteTrTable *byte2offset,
-	const cachedCharSeq *seq,
+	const Chars_holder *seq,
 	const BytewiseOpTable *bytewise_match_table
 );
 
@@ -48,12 +50,12 @@ int _shift_twobit_signature(
 
 int _get_twobit_signature(
 	TwobitEncodingBuffer *teb,
-	const cachedCharSeq *seq
+	const Chars_holder *seq
 );
 
 int _get_twobit_signature_at(
 	TwobitEncodingBuffer *teb,
-	const cachedCharSeq *seq,
+	const Chars_holder *seq,
 	const int *at,
 	int at_length
 );
@@ -61,15 +63,39 @@ int _get_twobit_signature_at(
 
 /* io_utils.c */
 
+int ExternalFilePtr_gets(
+	SEXP efp,
+	char *buf,
+	int buf_size,
+	int *EOL_in_buf
+);
+
+void ExternalFilePtr_rewind(SEXP efp);
+
+int ExternalFilePtr_puts(
+	SEXP efp,
+	const char *s
+);
+
+void ExternalFilePtr_putc(
+	SEXP efp,
+	int c
+);
+
 SEXP new_input_ExternalFilePtr(SEXP filepath);
 
-SEXP new_output_ExternalFilePtr(SEXP filepath, SEXP append);
+SEXP new_output_ExternalFilePtr(
+	SEXP filepath,
+	SEXP append,
+	SEXP compress,
+	SEXP compression_level
+);
 
-SEXP ExternalFilePtr_close(SEXP x);
+SEXP finalize_ExternalFilePtr(SEXP efp);
 
 int delete_trailing_LF_or_CRLF(
 	const char *buf,
-	int size
+	int buf_len
 );
 
 
@@ -100,16 +126,16 @@ char _RNAencode(char c);
 
 char _RNAdecode(char code);
 
-void _copy_CHARSXP_to_cachedCharSeq(
-	cachedCharSeq *dest,
+void _copy_CHARSXP_to_Chars_holder(
+	Chars_holder *dest,
 	SEXP src,
 	int start_in_src,
 	const int *lkup,
 	int lkup_length
 );
 
-SEXP _new_CHARSXP_from_cachedCharSeq(
-	const cachedCharSeq *x,
+SEXP _new_CHARSXP_from_Chars_holder(
+	const Chars_holder *x,
 	SEXP lkup
 );
 
@@ -137,15 +163,21 @@ SEXP _get_XStringSet_width(SEXP x);
 
 const char *_get_XStringSet_xsbaseclassname(SEXP x);
 
-cachedXStringSet _cache_XStringSet(SEXP x);
+XStringSet_holder _hold_XStringSet(SEXP x);
 
-int _get_cachedXStringSet_length(const cachedXStringSet *cached_x);
+int _get_length_from_XStringSet_holder(const XStringSet_holder *x_holder);
 
-cachedCharSeq _get_cachedXStringSet_elt(
-	const cachedXStringSet *cached_x,
+Chars_holder _get_elt_from_XStringSet_holder(
+	const XStringSet_holder *x_holder,
 	int i
 );
 
+XStringSet_holder _get_linear_subset_from_XStringSet_holder(
+	const XStringSet_holder *x_holder,
+	int offset,
+	int length
+);
+
 void _set_XStringSet_names(
 	SEXP x,
 	SEXP names
@@ -173,6 +205,20 @@ RoSeqs _new_RoSeqs_from_XStringSet(
 SEXP XStringSet_unlist(SEXP x);
 
 
+/* XStringSetList_class.c */
+
+XStringSetList_holder _hold_XStringSetList(SEXP x);
+
+int _get_length_from_XStringSetList_holder(
+	const XStringSetList_holder *x_holder
+);
+
+XStringSet_holder _get_elt_from_XStringSetList_holder(
+	const XStringSetList_holder *x_holder,
+	int i
+);
+
+
 /* xscat.c */
 
 SEXP XString_xscat(SEXP args);
@@ -188,6 +234,7 @@ SEXP fasta_info(
 	SEXP efp_list,
 	SEXP nrec,
 	SEXP skip,
+	SEXP seek_first_rec,
 	SEXP use_names,
 	SEXP lkup
 );
@@ -196,6 +243,7 @@ SEXP read_fasta_in_XStringSet(
 	SEXP efp_list,
 	SEXP nrec,
 	SEXP skip,
+	SEXP seek_first_rec,
 	SEXP use_names,
 	SEXP elementType,
 	SEXP lkup
@@ -211,13 +259,15 @@ SEXP write_XStringSet_to_fasta(
 SEXP fastq_geometry(
 	SEXP efp_list,
 	SEXP nrec,
-	SEXP skip
+	SEXP skip,
+	SEXP seek_first_rec
 );
 
 SEXP read_fastq_in_XStringSet(
 	SEXP efp_list,
 	SEXP nrec,
 	SEXP skip,
+	SEXP seek_first_rec,
 	SEXP use_names,
 	SEXP elementType,
 	SEXP lkup
@@ -351,9 +401,11 @@ void gtestsim(
 
 SEXP DNAStringSet_translate(
 	SEXP x,
-	SEXP base_codes,
+	SEXP skip_code,
+	SEXP dna_codes,
 	SEXP lkup,
-	SEXP skipcode
+	SEXP if_non_ambig,
+	SEXP if_ambig
 );
 
 
@@ -386,6 +438,15 @@ SEXP XString_inject_code(
 );
 
 
+/* unstrsplit_methods.c */
+
+SEXP XStringSetList_unstrsplit(
+	SEXP x,
+	SEXP sep,
+	SEXP seqtype
+);
+
+
 /* SparseList_utils.c */
 
 SEXP debug_SparseList_utils();
@@ -456,7 +517,7 @@ SEXP _MatchBuf_starts_asLIST(const MatchBuf *match_buf);
 
 SEXP _MatchBuf_ends_asLIST(const MatchBuf *match_buf);
 
-SEXP _MatchBuf_as_MIndex(const MatchBuf *match_buf);
+SEXP _MatchBuf_as_Ranges(const MatchBuf *match_buf);
 
 SEXP _MatchBuf_as_SEXP(
 	const MatchBuf *match_buf,
@@ -484,13 +545,19 @@ MatchBuf *_get_internal_match_buf();
 
 SEXP debug_MIndex_class();
 
-cachedMIndex _cache_MIndex(SEXP x);
+MIndex_holder _hold_MIndex(SEXP x);
 
-int _get_cachedMIndex_length(const cachedMIndex *cached_x);
+int _get_length_from_MIndex_holder(const MIndex_holder *x_holder);
 
-int _get_cachedMIndex_elt_width0(const cachedMIndex *cached_x, int i);
+int _get_width0_elt_from_MIndex_holder(
+	const MIndex_holder *x_holder,
+	int i
+);
 
-cachedIRanges _get_cachedMIndex_elt(const cachedMIndex *cached_x, int i);
+IRanges_holder _get_elt_from_MIndex_holder(
+	const MIndex_holder *x_holder,
+	int i
+);
 
 SEXP ByPos_MIndex_endIndex(
 	SEXP x_high2low,
@@ -517,16 +584,16 @@ void _init_bytewise_match_tables();
 const BytewiseOpTable *_select_bytewise_match_table(int fixedP, int fixedS);
 
 int _nmismatch_at_Pshift(
-	const cachedCharSeq *P,
-	const cachedCharSeq *S,
+	const Chars_holder *P,
+	const Chars_holder *S,
 	int Pshift,
 	int max_nmis,
 	const BytewiseOpTable *bytewise_match_table
 );
 
 int _nedit_for_Ploffset(
-	const cachedCharSeq *P,
-	const cachedCharSeq *S,
+	const Chars_holder *P,
+	const Chars_holder *S,
 	int Ploffset,
 	int max_nedit,
 	int loose_Ploffset,
@@ -535,8 +602,8 @@ int _nedit_for_Ploffset(
 );
 
 int _nedit_for_Proffset(
-	const cachedCharSeq *P,
-	const cachedCharSeq *S,
+	const Chars_holder *P,
+	const Chars_holder *S,
 	int Proffset,
 	int max_nedit,
 	int loose_Proffset,
@@ -578,8 +645,8 @@ SEXP XStringSet_dist_hamming(SEXP x);
 SEXP debug_match_pattern_boyermoore();
 
 int _match_pattern_boyermoore(
-	const cachedCharSeq *P,
-	const cachedCharSeq *S,
+	const Chars_holder *P,
+	const Chars_holder *S,
 	int nfirstmatches,
 	int walk_backward
 );
@@ -592,8 +659,8 @@ SEXP debug_match_pattern_shiftor();
 SEXP bits_per_long();
 
 void _match_pattern_shiftor(
-	const cachedCharSeq *P,
-	const cachedCharSeq *S,
+	const Chars_holder *P,
+	const Chars_holder *S,
 	int max_nmis,
 	int fixedP,
 	int fixedS
@@ -605,8 +672,8 @@ void _match_pattern_shiftor(
 SEXP debug_match_pattern_indels();
 
 void _match_pattern_indels(
-	const cachedCharSeq *P,
-	const cachedCharSeq *S,
+	const Chars_holder *P,
+	const Chars_holder *S,
 	int max_nmis,
 	int fixedP,
 	int fixedS
@@ -618,8 +685,8 @@ void _match_pattern_indels(
 SEXP debug_match_pattern();
 
 void _match_pattern_XString(
-	const cachedCharSeq *P,
-	const cachedCharSeq *S,
+	const Chars_holder *P,
+	const Chars_holder *S,
 	SEXP max_mismatch,
 	SEXP min_mismatch,
 	SEXP with_indels,
@@ -628,8 +695,8 @@ void _match_pattern_XString(
 );
 
 void _match_pattern_XStringViews(
-	const cachedCharSeq *P,
-	const cachedCharSeq *S,
+	const Chars_holder *P,
+	const Chars_holder *S,
 	SEXP views_start,
 	SEXP views_width,
 	SEXP max_mismatch,
@@ -671,7 +738,7 @@ SEXP XStringSet_vmatch_pattern(
 	SEXP with_indels,
 	SEXP fixed,
 	SEXP algorithm,
-	SEXP count_only
+	SEXP ms_mode
 );
 
 
@@ -948,7 +1015,7 @@ void _match_pdict_flanks_at(
 	int key0,
 	SEXP low2high,
 	HeadTail *headtail,
-	const cachedCharSeq *S,
+	const Chars_holder *S,
 	int tb_end,
 	int max_nmis,
 	int min_nmis,
@@ -959,7 +1026,7 @@ void _match_pdict_flanks_at(
 void _match_pdict_all_flanks(
 	SEXP low2high,
 	HeadTail *headtail,
-	const cachedCharSeq *S,
+	const Chars_holder *S,
 	int max_nmis,
 	int min_nmis,
 	MatchPDictBuf *matchpdict_buf
@@ -978,7 +1045,7 @@ SEXP build_Twobit(
 
 void _match_Twobit(
 	SEXP pptb,
-	const cachedCharSeq *S,
+	const Chars_holder *S,
 	int fixedS,
 	TBMatchBuf *tb_matches
 );
@@ -1030,7 +1097,7 @@ SEXP ACtree2_compute_all_flinks(SEXP pptb);
 
 void _match_tbACtree2(
 	SEXP pptb,
-	const cachedCharSeq *S,
+	const Chars_holder *S,
 	int fixedS,
 	TBMatchBuf *tb_matches
 );
@@ -1038,7 +1105,7 @@ void _match_tbACtree2(
 void _match_pdictACtree2(
 	SEXP pptb,
 	HeadTail *headtail,
-	const cachedCharSeq *S,
+	const Chars_holder *S,
 	int max_nmis,
 	int min_nmis,
 	int fixedP,
