@@ -31,8 +31,8 @@ static const R_CallMethodDef callMethods[] = {
 
 /* io_utils.c */
 	CALLMETHOD_DEF(new_input_ExternalFilePtr, 1),
-	CALLMETHOD_DEF(new_output_ExternalFilePtr, 2),
-	CALLMETHOD_DEF(ExternalFilePtr_close, 1),
+	CALLMETHOD_DEF(new_output_ExternalFilePtr, 4),
+	CALLMETHOD_DEF(finalize_ExternalFilePtr, 1),
 
 /* RoSeqs_utils.c */
 	CALLMETHOD_DEF(debug_RoSeqs_utils, 0),
@@ -56,11 +56,11 @@ static const R_CallMethodDef callMethods[] = {
 
 /* XStringSet_io.c */
 	CALLMETHOD_DEF(debug_XStringSet_io, 0),
-	CALLMETHOD_DEF(fasta_info, 5),
-	CALLMETHOD_DEF(read_fasta_in_XStringSet, 6),
+	CALLMETHOD_DEF(fasta_info, 6),
+	CALLMETHOD_DEF(read_fasta_in_XStringSet, 7),
 	CALLMETHOD_DEF(write_XStringSet_to_fasta, 4),
-	CALLMETHOD_DEF(fastq_geometry, 3),
-	CALLMETHOD_DEF(read_fastq_in_XStringSet, 6),
+	CALLMETHOD_DEF(fastq_geometry, 4),
+	CALLMETHOD_DEF(read_fastq_in_XStringSet, 7),
 	CALLMETHOD_DEF(write_XStringSet_to_fastq, 4),
 
 /* letter_frequency.c */
@@ -77,7 +77,7 @@ static const R_CallMethodDef callMethods[] = {
 	CALLMETHOD_DEF(XStringSet_two_way_letter_frequency_by_quality, 7),
 
 /* translate.c */
-	CALLMETHOD_DEF(DNAStringSet_translate, 4),
+	CALLMETHOD_DEF(DNAStringSet_translate, 6),
 
 /* replace_letter_at.c */
 	CALLMETHOD_DEF(XString_replace_letter_at, 6),
@@ -86,6 +86,9 @@ static const R_CallMethodDef callMethods[] = {
 /* inject_code.c */
 	CALLMETHOD_DEF(XString_inject_code, 4),
 
+/* unstrsplit_methods.c */
+	CALLMETHOD_DEF(XStringSetList_unstrsplit, 3),
+
 /* SparseList_utils.c */
 	CALLMETHOD_DEF(debug_SparseList_utils, 0),
 
@@ -221,11 +224,17 @@ void R_init_Biostrings(DllInfo *info)
 /* XStringSet_class.c */
 	REGISTER_CCALLABLE(_get_XStringSet_length);
 	REGISTER_CCALLABLE(_get_XStringSet_xsbaseclassname);
-	REGISTER_CCALLABLE(_cache_XStringSet);
-	REGISTER_CCALLABLE(_get_cachedXStringSet_length);
-	REGISTER_CCALLABLE(_get_cachedXStringSet_elt);
+	REGISTER_CCALLABLE(_hold_XStringSet);
+	REGISTER_CCALLABLE(_get_length_from_XStringSet_holder);
+	REGISTER_CCALLABLE(_get_elt_from_XStringSet_holder);
+	REGISTER_CCALLABLE(_get_linear_subset_from_XStringSet_holder);
 	REGISTER_CCALLABLE(_set_XStringSet_names);
 
+/* XStringSetList_class.c */
+	REGISTER_CCALLABLE(_hold_XStringSetList);
+	REGISTER_CCALLABLE(_get_length_from_XStringSetList_holder);
+	REGISTER_CCALLABLE(_get_elt_from_XStringSetList_holder);
+
 /* match_reporting.c */
 	REGISTER_CCALLABLE(_init_match_reporting);
 	REGISTER_CCALLABLE(_set_active_PSpair);
@@ -236,10 +245,10 @@ void R_init_Biostrings(DllInfo *info)
 	REGISTER_CCALLABLE(_reported_matches_asSEXP);
 
 /* MIndex_class.c */
-	REGISTER_CCALLABLE(_cache_MIndex);
-	REGISTER_CCALLABLE(_get_cachedMIndex_length);
-	REGISTER_CCALLABLE(_get_cachedMIndex_elt_width0);
-	REGISTER_CCALLABLE(_get_cachedMIndex_elt);
+	REGISTER_CCALLABLE(_hold_MIndex);
+	REGISTER_CCALLABLE(_get_length_from_MIndex_holder);
+	REGISTER_CCALLABLE(_get_width0_elt_from_MIndex_holder);
+	REGISTER_CCALLABLE(_get_elt_from_MIndex_holder);
 
 /* match_pattern_boyermoore.c */
 	REGISTER_CCALLABLE(_match_pattern_boyermoore);
